

//**************************************************************************************
//                          SPACE INVADER (PROGRAMME PROCESSING)
//                                         V1.7.4
// Ce programme est fait pour fonctionner avec une puce Picaxe ou Arduino et est utilisé 
// comme exemple de réalisation d'un jeu vidéo inspiré du cilte jeu SPACE INVADER
// des années 80.
//
// Programme réalisé par David SOUDER - Collège André Maurois - Neuilly-sur-Seine
// Plus d'informations sur le site de l'auteur www.duinoam.com 
//
//**************************************************************************************

//------------------------------------- Détails techniques------------------------------
/*

---------- Fonctionnement
 -Le jeu est géré automatiquement et n'a pas besoin de modifications
 -Il est cependant possible de modifier son apparence en modifiant les images
 présentent dans le dossier "data".
 -Il est possible d'utiliser pour jouer :
     -les flèches du clavier ;
     -une puce Picaxe (utilisation du port série)
     -une carte Picaxe (utilisation du port série)
     -une carte Arduino Uno ou Mega (utilisation du port série)
 ---------- Remarques sur l'utilisation du port série
  La communication est basée uniquement sur des caractères type :
     -'D'doit être transmis pour déplacer l'élément mobile vers la gauche
     -'G' doit être transmis pour déplacer l'élément mobile vers la gauche
  
  ----- Pour Picaxe
  C'est l'interface servant à programmer la puce elle-même qui est utilisée pour
  cela. Aucune installation supplémentaire n'est necessaire. Le matériel utilisé est
  celui qui doit nécessairement être installé pour programmer les puces Picaxe.
  
  ----- Pour Arduino
  C'est la même chose à part que l'interface est implentée sur la carte Arduino Uno ou Mega
  directement. Mais, le principe reste le même.
  
  ---------- Configuration du port série
  -Cofiguration propre au port série
      -Vitesse  : 9600 bauds
      -Données  : 8 bits (et pas 7)
      -Parité   : aucune (pas de vérification)
      -Bit d'arrêt : 1 seul (et pas 2)
      -Négative (et pas True)
      
Prérequis :
1-Pocéder un ordinateur avec Java
2-Pocéder la technologie Picaxe
    -Interface USB Picaxe installée
    -Drivers de l'interface installés
    -Programming Editor ou éqivalent installé
    suite au 3.1...
OU
2-Pocéder la technlogie Arduino
    -Carte Arduino Uno ou Mega installée

3.1-Copier/coller le programme ci-dessous dans Programming Editor
hsersetup B9600_4, %10                  ; *1 Configuration de l’UART Picaxe
        
        ;--------------------Proramme à placer dans Programming Editor---------------
        hsersetup B9600_4, %10                ; Configuration de l’UART Picaxe
        main:
          ici: if pin1=1 then aller_a_droite  ; Si bouton droit enfoncé, aller à...
          la:  if pin3=1 then aller_a_gauche  ; Si bouton gauche enfoné, aller à...
          goto main:

        aller_a_droite:
          hserout 0, ("D")                    ; *2 Transmission d’un mot par UART 
          sound 4, (50, 5)                    ; Permet de jouer un son
          goto la

        aller_a_gauche:
          hserout 0, ("G")
          sound 4, (60, 5)
          goto ici
        ;--------------------Fin du programme à placer dans Programming Editor--------
3.2-Téléverser le programme Picaxe dans la puce Picaxe





*/

//===================================== LIBRAIRIES =====================================
import processing.serial.*;              // Import de la librairie Serial (non implicite av Processing)


//===================================== VARIABLES/CONSTANTES ===========================
//---------- COMPTEURS
int cnt     = 0;                          // Augmente de 1 à chaque passage de draw() (RAZ par calculerObjets)
int cntMax  = 5;                          // Nombre de boucles avant recalcul des positions d'objets
int nbrVies = 8;                          // Diminue de 1 à chaque perte de vie
int attenteNouvelleColision=1;            // Est placé à 0 quand une colision est en cours et est déjà décomptée
int nbrPts  = 0;                          // Nombre de points gagnés par le joueur
int finPartie = 0;                        // Est placé à 1 quand la partie est finie
int defilementPartieAutorise = 0;         // Vaut 1 si la partie est autorisée à se dérouler
int scanPortsAFaire = 1;                  // Vaut 1 si un scan des ports série doit être effectué
int constructionPortsAFaire = 1;          // Vaut 1 si une canstruction de port doit être faite
String[] portsOuverts={"Aucun"};          // Tableau devant contenir les différents noms de ports COM1,COM2... après scan
String[] portsOuvertsVerif={"Aucun"};    // IDEM mais permet une comparaison si vérif
                                             //Je vais utiliser : 
                                                 // portsOuverts[0]="COM1"; pour remplacer "Aucun" par COM1
                                                 // portsOuverts = append (portsOuverts, "COM3"); pour ajouter un port en fin de liste
                                                 // portsOuverts = splice (portsOuverts , "COM2", 2-1); pour insérer un élément
                                                 // portsOuverts = shorten(portsOuverts); pour retirer l'élément de fin de liste
int nbrPortsOuverts = 0;                  // Contiendra le nombre de ports ouverts sur l'ordinateur
int nbrPortsOuvertsVerif = 0;             // IDEM mais permet une comparaison si vérif
int intro  = 1;                           // Vaut 0 quand l'intro est faite                                          
int config = 0;                           // Vaut 1 quand la config est autorisée
int finBat    = 0;                        // Est placé à 1 quand il n'y a plus qu'un seul bâtiment ou aucun
int roi       = 0;                        // Est placé à 1 quand un roi est constitué

//---------- PORT SERIE ---------------
Serial[] monPort;                         // Déclaration d'un "Objet Port série"
char[] caracRecu = {0,0,0,0,0,0,0,0,0,0};                         // Doit contenir le dernier caractère reçu pour les i ports
int  chiffreRecu        =  0;             // Doit contenir le chiffre [0à9] type integer
//---------- FENETRE ------------------
int largFen             = 800;            // Largeur fenêtre de jeu
int hautFen             = 550;            // Hauteur fenêtre de jeu

//---------- DESSINS ------------------
//----- Grille
int nbrCol           = 10;
int nbrLig           = 10;
int nbrSepCol        = nbrCol-1;        // Nombre de séparateurs de colonnes
int nbrSepLig        = nbrLig-1;        // Nombre de séparateurs de lignes
int largCol          = largFen/nbrCol;  // Largeur d'une colonne
int hautLig          = hautFen/nbrLig;  // Hauteur d'une colone
//----- Objets SI
int posVaisseauMere = 4;                // Par défaut, le vaisseau mère est au milieu
int deplacerVaisseauADroiteParTouche=0; // Vaut 1 si un déplacement est demandé par la touche flèche droite du clavier
int deplacerVaisseauAGaucheParTouche=0; // Vaut 1 si un déplacement est demandé par une touche flèche gauche du clavier
int deplacerVaisseauADroiteParSerial=0; // Vaut 1 si un déplacement est demandé par le port série avec le caractère D
int deplacerVaisseauAGaucheParSerial=0; // Vaut 1 si un déplacement est demandé par le port série avec le caractère G
int deplacerVaisseauADroite = 0;        // Vaut 1 s'il faut déplacer d'un crant à droite le vaisseau
int deplacerVaisseauAGauche = 0;        // Vaut 1 s'il faut déplacer d'un crant à gauche le vaisseau
int deplacementRealiseParTouche=0;      // Vaut 1 si le déplacement d'un crant à été réalisé du fait d'un appui sur une touche
//----- Déclaration des objets PImage (= vaisseau; robot...)
PImage robot01 ;
PImage vaisseauMere;
PImage vaisseauMere10;
PImage vaisseauMere09;
PImage vaisseauMere08;
PImage vaisseauMere07;
PImage vaisseauMere06;
PImage vaisseauMere05;
PImage vaisseauMere04;
PImage vaisseauMere03;
PImage vaisseauMere02;
PImage vaisseauMere01;
PImage vaisseauMere00;
PImage vaisseauMereMoins1;
PImage vaisseauMereMoins2;
PImage vaisseauMereMoins3;
PImage vaisseauMereMoins4;
PImage vaisseauMereMoins5;
PImage vaisseauMereMoins6;
PImage vaisseauMereMoins7;
PImage vaisseauMereMoins8;
PImage vaisseauMereMoins9;
PImage vaisseauMereMoins10;
PImage ArrierePlan;
//----- Déclaration des images liées à la page de congiguration
PImage cocheVerte ;
PImage cocheOrange ;
PImage exclamationOrange;
PImage croixRouge;
PImage carreVert;
PImage cocheGrise;
PImage glogo;
PImage logo10;

int[][] posObj   = {      // Un 1 est placé pour déclarer un objet dans la matrice (constituée des lignes et colonnes)
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0,0,0},
}; 
//----- Base
// Liste de tous les bâtements de la base
int batimentsBase[]={10,10,10,10,10,10,10,10,10,10};                // Un 1 est placé à 0 quand le bâtiment correspondant est 
//Batiments                                                         // complètement détruit
int nbrBat=10;
int indiceBat=10-1;                                                 // Le bâtiment 9 est le 10ème !

//----- POLICES D'ECRITURE
PFont font01;            // Doit ensuite être initialisé dans le setup()
PFont font02;
PFont font03;
PFont font04;
PFont font05;      

//---------- Couleurs
color noir = color(0,0,0);



//===================================== SETUP ==========================================
void setup(){
  initialiserFenetre();
  initialiserPolices();
  initialiserObjets();
  initialiserImages();
}//## Fin setup()  
            
//===================================== DRAW ===========================================
void draw(){
  incrementerCompteurs();                // Compte le nombre de boucles : résuslat dans la variable cnt
  scannerPorts();                        // Permet de remplir un tableau avec les ports séries ouverts
  initialiserPorts();                    // Permet de créér l'objet port série avec le n°1, 2 ou 3

//-------------------------------------- // Page d'intro
if(intro==1){ dessinerPageIntro(); }     // Permet de gérer complètement l'affichage la page d'intro. Passage de relais géré en intene

//-------------------------------------- // Page de gestion de la config
if(config==1){ dessinerPageConfig(); }   // Permet de gérer complètement la configration des ports

//-------------------------------------- // Défilement de jeu
if (defilementPartieAutorise==1){
  nettoyerFenetre();                     // On commence par effacer tout à l'écran
  dessinerArrierePlan();                 // Permet de dessiner un arrière-plan
  lectureCracterePortSerie();            // Permet de lire le caractère présent dans le buffer du port série
  claculCaractereSerie();                // On modifie des variables en fonction du caractère recu
  calculerDeplacementVaisseauParSerial();// On calcule un déplacement pour le vaisseau mère
  lectureToucheClavier();                // Permet de lire le caractère au clavier
  calculerDeplacementVaisseauParTouche();// Permet d'incrémenter ou décrémenter la position du vaisseau en fct du clavier...
  dessinerVaisceauMere();                // Pemet de dessiner le vaisseau mère en tenant compte du placement calculé
  calculerMatriceObjets();               // Calcul le placement des objets mobiles type robots
  dessinerObjets();                      // Dessine les objets mobiles type robots
  detecterColosionsVaisseauMere();       // Détermine si une colision a lieu entre le vaisseau mère et un robot (gain de points)
  detecterColisionBase();                // Détermine si une colision a lieu entre la base et un robot (mauvois signe)
  dessinerBarreDesPoints();              // Dessine la barre des points [Dans fichier D_PTS]
  ecrireNombreDePoints();                // Ecrit le nombre de points   [Dans fichier D_PTS]
  //dessinerLesVies();                   // Dessine le nombre de vies restantes
  dessinerLaBase();                      // Dessine la base en tenant avec les images du tube de20 images base
  dessinerGrille();                      // Dessine une grille à l'écran 
  detecterLaFinDePartie();               // Détermine si c'est la fin de la partie
}
  dessinerLaFinDePartie();               // Dessine tous les éléments de fin de partie si besoin
  gererLeReload();                       // Pemet de replacer toutes les variables dejeu à 0
 
}//## Fin draw()
